import pytest
from django.urls import reverse

from .test_app.library.factories import UserFactory
from .utils import parse_sidemenu, parse_topmenu, parse_usermenu


@pytest.mark.django_db
def test_side_menu(admin_client, custom_richmin_settings):
    """
    All menu tweaking settings work as expected
    """
    url = reverse('admin:index')

    response = admin_client.get(url)

    assert parse_sidemenu(response) == {
        'Administration': ['/en/admin/admin/logentry/'],
        'Authentication and Authorization': [
            '/en/admin/auth/group/',
            '/en/admin/auth/user/',
        ],
        'Books': [
            '/en/admin/books/author/',
            '/en/admin/books/book/',
            '/en/admin/books/genre/',
        ],
        'Global': ['/en/admin/'],
        'Loans': [
            '/make_messages/',
            '/en/admin/loans/bookloan/',
            '/en/admin/loans/library/',
            '/en/admin/loans/bookloan/custom_view',
        ],
    }

    custom_richmin_settings['hide_models'] = ['auth.user']
    response = admin_client.get(url)

    assert parse_sidemenu(response) == {
        'Global': ['/en/admin/'],
        'Authentication and Authorization': ['/en/admin/auth/group/'],
        'Books': [
            '/en/admin/books/author/',
            '/en/admin/books/book/',
            '/en/admin/books/genre/',
        ],
        'Loans': [
            '/make_messages/',
            '/en/admin/loans/bookloan/',
            '/en/admin/loans/library/',
            '/en/admin/loans/bookloan/custom_view',
        ],
        'Administration': ['/en/admin/admin/logentry/'],
    }


@pytest.mark.django_db
def test_permissions_on_custom_links(client, custom_richmin_settings):
    """
    we honour permissions for the rendering of custom links
    """
    user = UserFactory()
    user2 = UserFactory(permissions=('books.view_book',))

    url = reverse('admin:index')

    custom_richmin_settings['custom_links'] = {
        'books': [{
            'name': 'Make Messages',
            'url': 'make_messages',
            'icon': 'fa-comments',
            'permissions': ['books.view_book'],
        }]
    }

    client.force_login(user)
    response = client.get(url)
    assert parse_sidemenu(response) == {'Global': ['/en/admin/']}

    client.force_login(user2)
    response = client.get(url)
    assert parse_sidemenu(response) == {
        'Global': ['/en/admin/'],
        'Books': ['/make_messages/', '/en/admin/books/book/'],
    }


@pytest.mark.django_db
def test_top_menu(admin_client, custom_richmin_settings):
    """
    Top menu renders out as expected
    """
    url = reverse('admin:index')

    custom_richmin_settings['topmenu_links'] = [
        {
            'name': 'Home',
            'url': 'admin:index',
            'permissions': ['auth.view_user']
        },
        {
            'name': 'Support',
            'url': 'https://github.com/mymy47/django-richmin',
            'new_window': True,
        },
        {
            'model': 'auth.User'
        },
        {
            'app': 'books'
        },
    ]

    response = admin_client.get(url)

    assert parse_topmenu(response) == [
        {
            'name': 'Home',
            'link': '/en/admin/'
        },
        {
            'name': 'Support',
            'link': 'https://github.com/mymy47/django-richmin',
        },
        {
            'name': 'Users',
            'link': '/en/admin/auth/user/'
        },
        {
            'name': 'Books',
            'link': '#',
            'children': [
                {
                    'name': 'Genres',
                    'link': '/en/admin/books/genre/'
                },
                {
                    'name': 'Books',
                    'link': '/en/admin/books/book/'
                },
                {
                    'name': 'Authors',
                    'link': '/en/admin/books/author/'
                },
            ],
        },
    ]


@pytest.mark.django_db
def test_user_menu(admin_user, client, custom_richmin_settings):
    """
    The User menu renders out as expected
    """
    url = reverse('admin:index')

    custom_richmin_settings['usermenu_links'] = [
        {
            'name': 'Home',
            'url': 'admin:index',
            'permissions': ['auth.view_user']
        },
        {
            'name': 'Support',
            'url': 'https://github.com/mymy47/django-richmin',
            'new_window': True,
        },
        {
            'model': 'auth.User'
        },
    ]

    client.force_login(admin_user)
    response = client.get(url)

    assert parse_usermenu(response) == [
        {
            'link': '/en/admin/password_change/',
            'name': 'Change password'
        },
        {
            'link': '/en/admin/logout/',
            'name': 'Log out'
        },
        {
            'link': '/en/admin/',
            'name': 'Home'
        },
        {
            'link': 'https://github.com/mymy47/django-richmin',
            'name': 'Support',
        },
        {
            'link': '/en/admin/auth/user/',
            'name': 'Users'
        },
        {
            'link': '/en/admin/auth/user/{}/change/'.format(admin_user.pk),
            'name': 'See Profile',
        },
    ]



from collections import defaultdict

from bs4 import BeautifulSoup, Tag
from faker import Faker

fake = Faker()


def parse_sidemenu(response):
    """
    Convert the side menu to a dict keyed on app name, containing a list of links
    """
    menu = defaultdict(list)
    current_app = 'Global'
    soup = BeautifulSoup(response.content, 'html.parser')

    for li in soup.find(id='richy-sidebar').find('ul').find_all('li'):
        if 'nav-header' in li['class']:
            current_app = li.text.strip()

        elif 'nav-item' in li['class']:
            href = li.find('a')
            menu[current_app].append(href['href'] if href else None)

    return menu


def parse_topmenu(response):
    """
    Convert the top menu to a list of dicts representing menus, items with submenus will have key 'children'
    """
    menu = []
    soup = BeautifulSoup(response.content, 'html.parser')

    for li in soup.find(id='richy-navbar').find('ul').find_all('li'):
        anchor = li.find('a')

        # Skip brand link and menu button
        if type(anchor.contents[0]) is Tag:
            continue

        item = {'name': anchor.text.strip(), 'link': anchor['href']}
        dropdown = li.find('div', class_='dropdown-menu')
        if dropdown:
            item['children'] = [{'name': a.text.strip(), 'link': a['href']} for a in dropdown.find_all('a')]

        menu.append(item)

    return menu


def parse_usermenu(response):
    """
    Convert the user menu to a list of dicts representing menus
    """
    menu = []
    soup = BeautifulSoup(response.content, 'html.parser')

    for link in soup.find(id='richy-usermenu').find_all('a'):
        item = {'name': link.text.strip(), 'link': link['href']}
        menu.append(item)

    return menu



from random import choice

from django.contrib.auth.models import Group, User
from django.core.management import BaseCommand

from ...models import Author, Book, Genre
from ....factories import AuthorFactory, BookFactory, BookLoanFactory, GroupFactory, LibraryFactory, UserFactory
from ....loans.models import BookLoan, Library


class Command(BaseCommand):
    """
    A management script for resetting all data
    """

    def handle(self, *args, **options):
        for Model in [Group, User, Library, Author, Book, BookLoan, Genre]:
            Model.objects.all().delete()

        library = LibraryFactory()
        UserFactory(
            username='test@test.com',
            email='test@test.com',
            password='test',
            is_superuser=True,
        )

        users = UserFactory.create_batch(2, is_staff=False)

        GroupFactory.create_batch(5)

        for author in AuthorFactory.create_batch(10):
            for book in BookFactory.create_batch(5, author=author, library=library):
                BookLoanFactory(book=book, borrower=choice(users))

        self.stdout.write('All Data reset')



import pytest
from bs4 import BeautifulSoup
from django.urls import reverse

from richmin.settings import CHANGEFORM_TEMPLATES
from richmin.templatetags.richmin import get_sections
from .test_app.library.books.admin import BookAdmin
from .test_app.library.factories import BookFactory, UserFactory


@pytest.mark.django_db
def test_update_site_logo(admin_client, custom_richmin_settings):
    """
    We can add a site logo, and it renders out
    """
    url = reverse('admin:index')

    custom_richmin_settings['site_logo'] = 'books/img/logo.png'
    response = admin_client.get(url)
    soup = BeautifulSoup(response.content, 'html.parser')

    assert soup.find('a', class_='brand-link').find('img')['src'] == '/static/books/img/logo.png'


@pytest.mark.django_db
def test_update_login_logo(client, custom_richmin_settings):
    """
    We can add a login logo, and it renders out
    """
    url = reverse('admin:login')

    custom_richmin_settings['login_logo'] = 'books/img/logo-login.png'
    response = client.get(url)
    soup = BeautifulSoup(response.content, 'html.parser')

    assert soup.find('div', class_='login-logo').find('img')['src'] == '/static/books/img/logo-login.png'


@pytest.mark.django_db
@pytest.mark.parametrize('config_value,template', [(k, v) for k, v in CHANGEFORM_TEMPLATES.items()])
def test_changeform_templates(config_value, template, admin_client, custom_richmin_settings):
    """
    All changeform config values use the correct templates
    """
    custom_richmin_settings['changeform_format'] = config_value
    book = BookFactory()
    url = reverse(
        'admin:books_book_change', args=(book.pk,)
    )

    response = admin_client.get(url)
    templates_used = [t.name for t in response.templates]

    assert template in templates_used


@pytest.mark.django_db
def test_changeform_template_override(admin_client, custom_richmin_settings):
    """
    We can set a global template, and override it per model
    """
    custom_richmin_settings.update({
        'changeform_format': 'vertical_tabs',
        'changeform_format_overrides': {
            'books.book': 'carousel'
        }
    })

    user = UserFactory()
    book = BookFactory()

    books_url = reverse(
        'admin:books_book_change', args=(book.pk,)
    )
    users_url = reverse(
        'admin:auth_user_change', args=(user.pk,)
    )

    response = admin_client.get(books_url)
    templates_used = [t.name for t in response.templates]

    assert CHANGEFORM_TEMPLATES['carousel'] in templates_used

    response = admin_client.get(users_url)
    templates_used = [t.name for t in response.templates]

    assert CHANGEFORM_TEMPLATES['vertical_tabs'] in templates_used


@pytest.mark.django_db
def test_changeform_template_default(admin_client, custom_richmin_settings):
    """
    The horizontal_tabs template is used by default
    """
    assert custom_richmin_settings['changeform_format'] == 'horizontal_tabs'
    book = BookFactory()

    books_url = reverse(
        'admin:books_book_change', args=(book.pk,)
    )

    response = admin_client.get(books_url)
    templates_used = [t.name for t in response.templates]

    assert CHANGEFORM_TEMPLATES['horizontal_tabs'] in templates_used


@pytest.mark.django_db
def test_changeform_single(admin_client, monkeypatch):
    """
    The single template is used when the modeladmin has no fieldsets, or inlines
    """
    book = BookFactory()
    books_url = reverse(
        'admin:books_book_change', args=(book.pk,)
    )
    monkeypatch.setattr(BookAdmin, 'fieldsets', None)
    monkeypatch.setattr(BookAdmin, 'inlines', [])

    response = admin_client.get(books_url)

    templates_used = [t.name for t in response.templates]
    assert CHANGEFORM_TEMPLATES['single'] in templates_used


@pytest.mark.django_db
@pytest.mark.parametrize('order', [('Book loans', 'general', 'other'), ('other', 'Book loans', 'general')])
def test_changeform_section_ordering(change_form_context, order):
    """
    We respect the given order for sections
    """
    admin_form = change_form_context['adminform']
    inline_formsets = change_form_context['inline_admin_formsets']

    admin_form.model_admin.richmin_section_order = order
    fieldsets = get_sections(admin_form, inline_formsets)

    assert tuple(x.name for x in fieldsets) == order



from richmin.settings import get_search_model_string


def test_get_search_model_string():
    # model name is always lower case
    assert get_search_model_string('books.Book') == 'books.book'
    assert get_search_model_string('books.book') == 'books.book'
    # the app name gets never touched
    assert get_search_model_string('Books.Book') == 'Books.book'
    assert get_search_model_string('BookShelf.book') == 'BookShelf.book'



# Generated by Django 3.2.13 on 2022-07-15 10:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("books", "0006_auto_20220327_0814"),
    ]

    operations = [
        migrations.AlterField(
            model_name="book",
            name="last_print",
            field=models.DateField(auto_now_add=True),
        ),
    ]



import json
from unittest.mock import MagicMock

import pytest
from django.contrib.admin.models import CHANGE, LogEntry

from richmin.templatetags import richmin


@pytest.mark.django_db
def test_app_is_installed(settings):
    """
    Returns True if an app is under INSTALLED_APPS, False otherwise
    """
    app = 'test_app'

    assert richmin.app_is_installed(app) is False

    settings.INSTALLED_APPS.append(app)

    assert richmin.app_is_installed(app) is True


@pytest.mark.django_db
def test_action_message_to_list(admin_user):
    """
    We can generate a list of messages from a log entry object
    """
    message = [
        {
            'changed': {
                'fields': ['Owner', 'Text', 'Pub date', 'Active']
            }
        },
        {
            'added': {
                'name': 'choice',
                'object': 'More random choices'
            }
        },
        {
            'deleted': {
                'name': 'choice',
                'object': 'Person serious choose tea'
            }
        },
    ]
    log_entry = LogEntry.objects.create(user=admin_user, action_flag=CHANGE, change_message=json.dumps(message))
    assert richmin.action_message_to_list(log_entry) == [
        {
            'msg': 'Changed Owner, Text, Pub date and Active.',
            'icon': 'edit',
            'colour': 'blue'
        },
        {
            'msg': 'Added choice “More random choices”.',
            'icon': 'plus-circle',
            'colour': 'success'
        },
        {
            'msg': 'Deleted “Person serious choose tea”.',
            'icon': 'trash',
            'colour': 'danger'
        },
    ]


def test_style_bold_first_word():
    """
    Adds <strong> HTML element wrapping first word given a sentence
    """
    message = 'The bomb has been planted'

    assert richmin.style_bold_first_word(message) == '<strong>The</strong> bomb has been planted'


@pytest.mark.django_db
@pytest.mark.parametrize(
    'case,test_input,field,expected',
    [
        (1, MagicMock(avatar='image.jpg'), 'avatar', 'image.jpg'),
        (2, MagicMock(avatar='image.jpg'), lambda u: u.avatar, 'image.jpg'),
        (3, MagicMock(avatar=MagicMock(url='image.jpg')), 'avatar', 'image.jpg'),
    ],
)
def test_get_user_avatar(case, test_input, field, expected, custom_richmin_settings):
    """
    We can specify the name of a charfield or imagefield on our user model, or a callable that receives our user
    """
    custom_richmin_settings['user_avatar'] = field
    assert richmin.get_user_avatar(test_input) == expected



# Generated by Django 2.2.15 on 2020-10-14 12:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("loans", "0001_initial"),
        ("books", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="book",
            name="library",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="books",
                to="loans.Library",
            ),
        ),
    ]



# Generated by Django 2.2.15 on 2020-10-15 11:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("books", "0002_book_library"),
    ]

    operations = [
        migrations.AlterField(
            model_name="book",
            name="isbn",
            field=models.CharField(
                help_text=(
                    '13 Character <a href="https://www.isbn-international.org/content/what-isbn">ISBN number</a>',
                ),
                max_length=13,
                verbose_name="ISBN",
            ),
        ),
    ]



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

setup(
    name='django-richmin',
    url='https://github.com/mymy47/django-richmin',
    version='0.1.4',
    license='MIT License',
    packages=find_packages(),
    include_package_data=True,
    description='Customized lte-bootstrap django admin theme with some useful tools',
    long_description=README,
    long_description_content_type='text/markdown',
    python_requires='>=3.6',
    author='my47',
    author_email='mymy47@gmail.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Framework :: Django :: 4.0',
        'Framework :: Django :: 4.1',
        'Framework :: Django :: 4.2',
        'Topic :: Software Development',
        'Topic :: Software Development :: User Interfaces',
    ],
)
